# Limit V1.1 by Black Wolf <b-wolf@bigfoot.com>
#
# This script is based on limit by Chutt.
#
# This script will set limit on every channel, except
# on channels flagged with -l
#
# Added commands:
#	.limit on			Activates the script (default)
#	.limit off			Deactivates the script
#	.limit check		Forces the script to check limit
#	.limit info			Show current settings
#	.limit set time	<n>	Set <n> minutes between checks
#	.limit set limit	<n>	Set the limit to <n> over current

#
# Initial variables
#
# minutes between checks
set LimitTimer 5

# set the limit to ?? over the current
set LimitIncr 10

# Set this to 1 if you don't want
# Limit to start automaticly
set LimitStart 0



# Internal Variables
set LimitVersion "Limit v1.1"

#
# Bindings
#
bind dcc n limit procLimitParser

#
# Binded commands
#
proc procLimitParser {handle idx args} {
	global LimitIncr LimitTimer LimitVersion
	foreach arg [string tolower $args] {
		if ![string compare "on" [lindex $arg 0]] {
			if [procLimitOn] {
				putdcc $idx "$LimitVersion activated"
			} else {
				putdcc $idx "$LimitVersion is allready activated"
			}
		} elseif ![string compare "off" [lindex $arg 0]] {
			if [procLimitOff] {
				putdcc $idx "$LimitVersion deactivated"
			} else {
				putdcc $idx "$LimitVersion is allready deactivated"
			}
		} elseif ![string compare "check" [lindex $arg 0]] {
			putdcc $idx "Checking limit"
			procLimitCheck
		} elseif ![string compare "info" [lindex $arg 0]] {
			foreach timerrec [string tolower [timers]] {
				if ![string compare "proclimitloop" [lindex $timerrec 1]] {
					putdcc $idx " Current Timer ID: [lindex $timerrec 2] (expires in [lindex $timerrec 0] minutes)"
					putdcc $idx "Check limit every: $LimitTimer minutes"
					putdcc $idx "  Limit is set to: $LimitIncr over the current"
				}
			}
		} elseif ![string compare "set" [lindex $arg 0]] {
			if {![string compare "time" [lindex $arg 1]] && [procLimitIsNumber [lindex $arg 2]]} {
				set LimitTimer [string range $arg [string wordstart $arg [string length $arg]] end]
				putdcc $idx "Limit will be checked every $LimitTimer minutes"
			} elseif {![string compare "limit" [lindex $arg 1]] && [procLimitIsNumber [lindex $arg 2]]} {
				set LimitIncr [string range $arg [string wordstart $arg [string length $arg]] end]
				putdcc $idx "Limit is set to $LimitIncr over the current"
			} else {
				putdcc $idx "Usage: limit set <time|limit> <value>"
			}
		} else {
			putdcc $idx "Usage: limit <on|off|check|set|info>"
		}
	}
}


#
# Internal procedures
#
proc procLimitOn {} {
	global LimitTimer LimitStart
	if {$LimitStart == 0} {
		set LimitStart 1
		procLimitCheck
		timer $LimitTimer procLimitLoop
		return 1
	} else {
		return 0
	}
}

proc procLimitOff {} {
	global LimitStart
	if {$LimitStart == 1} {
		foreach timerrec [string tolower [timers]] {
			if ![string compare "proclimitloop" [lindex $timerrec 1]] {
				killtimer [lindex $timerrec 2]
			}
		}
		set LimitStart 0
		return 1
	} else {
		return 0
	}
}

proc procLimitCheck {} {
	global LimitIncr LimitTimer
	foreach chan [procLimitChanlist] {
		set users 0
		set modes [getchanmode $chan]
		if [string match "*l*" $modes] {
			set cur_limit [string trimleft $modes "+klipsmtna "]
		} else {
			set cur_limit 0
		}
		foreach i [chanlist $chan] {
			set users [expr $users + 1]
		}
		set limit [expr (($users / ($LimitIncr / 2)) * ($LimitIncr / 2))]
		incr limit $LimitIncr
		if { $limit != $cur_limit } {
			pushmode $chan +l $limit
		}
	}
}

proc procLimitIsNumber {test} {
	foreach i [split $test {}] {
		if ![string match \[0-9\] $i] {
			return 0
		}
	}
	return 1
}

proc procLimitChanlist {} {
	foreach chan [string tolower [channels]] {
		set modes [string trim [lindex [channel info $chan] 0] "+kipsmtna"]
		if {[string compare "-kl" $modes] && [string compare "-l" $modes]} {
			lappend LimitChanList $chan
		}
	}
	if [info exists LimitChanList] {
		return $LimitChanList
	} else {
		return ""
	}
}

proc procLimitLoop {} {
	global LimitTimer
	procLimitCheck
	timer $LimitTimer procLimitLoop
}

procLimitOn
putlog "$LimitVersion by Black Wolf (based on Chutt's limitv1)"
